"""DAG implementation."""

import asyncio
from graphlib import TopologicalSorter
from typing import AsyncIterable, Generic, Iterable, TypeVar

NODE = TypeVar("NODE")


class Graph(Generic[NODE], TopologicalSorter):
    """graphlib.TopologicalSorter wrapper."""

    def __init__(self, graph=None):
        super().__init__(graph)
        self.wake = asyncio.Event()

    def get_ready(self):
        self.wake.clear()
        return super().get_ready()

    def done(self, *nodes: NODE):
        super().done(*nodes)
        self.wake.set()

    def static_order(self) -> Iterable[NODE]:
        """Returns an iterable of nodes in a topological order."""

        while self.is_active():
            nodes = self.get_ready()
            yield from nodes
            self.done(*nodes)

    async def nodes(self) -> AsyncIterable[NODE]:
        """Generator of nodes coming ready in topographical order."""

        while self.is_active():
            for node in self.get_ready():
                yield node
            await self.wake.wait()
