from contextlib import contextmanager
from time import perf_counter
from typing import Generator

from luna_analytics_manager.monitoring.monitoring import BaseMonitoringPoint


class Storage:
    """
    Container for monitoring points

    Attributes:
        name: storage name
        enable: enable or not monitoring point storing
        _values: map - keys - series name, values - list monitoring points
    """

    def __init__(self, name: str, enable: bool = True):
        self._values: dict = {}
        self.name = name
        self.enable = enable

    @contextmanager
    def popValues(self) -> Generator[dict[str, list[BaseMonitoringPoint]], None, None]:
        """pop points from  the storage"""
        yield self._values
        # if correct processing points, remove them from the store
        self._values = {}

    def append(self, point: BaseMonitoringPoint):
        """Add point to the storage"""
        self._values.setdefault(point.name, []).append(point)

    @contextmanager
    def monitoring(self, point):
        """
        Collect monitoring point ti storage (a event duration measurement).
        """
        if not self.enable or point is None:
            yield point
            return
        start = perf_counter()
        try:
            yield point
        finally:
            point.executionTime = perf_counter() - start
            self.append(point)


NODES_STORAGE = Storage("node_execution", False)
FRAMES_STORAGE = Storage("frame_monitoring", False)
