from lunavl.sdk.estimators.body_estimators.body_attributes import BodyAttributes
from lunavl.sdk.trackengine.structures import BodyTrack, FaceTrack


def convertColor(sdkColor: str) -> str:
    """Converts sdk color to handlers one."""
    if sdkColor == "unknown":
        return "undefined"
    elif sdkColor == "grey":
        # en-gb to en-us
        return "gray"
    return sdkColor


def convertColors(sdkColors: list[str]):
    """Converts sdk colors to handlers one."""
    result = []
    for sdkColor in sdkColors:
        result.append(convertColor(sdkColor))
    return result


def unknownToUndefined(value: str):
    """Converts unknown values to undefined"""
    if value == "unknown":
        return "undefined"
    return value


class MinimalTrack:
    """Minimal track."""

    def __init__(self, track: FaceTrack | BodyTrack):
        self.firstFrame = track.firstFrame
        self.bbox = track.bbox
        self.score = track.detection.boundingBox.score if track.detection else None

    def asDict(self):
        """Build results in API format."""

        return buildTrack(self)


def buildTrack(track: FaceTrack | BodyTrack | MinimalTrack):
    """Build track in API format."""

    if isinstance(track, MinimalTrack):
        return {"bbox": track.bbox.asDict(), "score": track.score}

    return {"bbox": track.bbox.asDict(), "score": track.detection.boundingBox.score if track.detection else None}


def buildBodyAttributes(attributes: BodyAttributes):
    """Build body attributes in API format."""

    return {
        "basic_attributes": {
            "apparent_age": int(attributes.apparentAge),
            "apparent_gender": attributes.apparentGender.predominantGender.value,
        },
        "upper_body": {
            "headwear": {
                "state": attributes.headwear.predominantState.value,
                "apparent_color": convertColor(attributes.headwear.apparentColor.value),
            },
            "sleeve": {"length": unknownToUndefined(attributes.sleeve.predominantState.name.lower())},
            "upper_clothing": {"colors": [convertColor(color.value) for color in attributes.outwearColor.colors]},
        },
        "lower_body": {
            "shoes": {"apparent_color": convertColor(attributes.shoes.apparentColor.value)},
            "lower_garment": {
                "type": unknownToUndefined(attributes.lowerGarment.type.value),
                "colors": [convertColor(color.value) for color in attributes.lowerGarment.colors],
            },
        },
        "accessories": {"backpack": {"state": attributes.backpack.predominantState.value}},
    }
