"""Human tracking analytics setup."""

from .aggregation import (
    NodeBodyAttributes as AggNodeBodyAttributes,
    NodeBodyBestshot as AggNodeBodyBestshot,
    NodeBodyDescriptor as AggNodeBodyDescriptor,
    NodeBodyShots as AggNodeBodyShots,
    NodeDeepfake as AggNodeDeepfake,
    NodeFaceBasicAttributes as AggNodeFaceBasicAttributes,
    NodeFaceBestshot as AggNodeFaceBestshot,
    NodeFaceDescriptor as AggNodeFaceDescriptor,
    NodeFaceShots as AggNodeFaceShots,
    NodeLivenessV1 as AggNodeLivenessV1,
    NodeOverview,
    _NodeAggregationPreserver,
)
from .estimations import (
    NodeAGS,
    NodeEmotions,
    NodeEyes,
    NodeFaceWarpQuality,
    NodeGaze,
    NodeGlasses,
    NodeHeadpose,
    NodeLandmarks5,
    NodeMask,
    NodeMouth,
    _NodeEstimationPreserver,
)
from .event_policy import NodeHumanTrackingEventPolicy
from .models import Params, Targets

EXTRA_NODES = {
    NodeOverview,
    NodeLandmarks5,
    NodeHeadpose,
    NodeAGS,
    NodeFaceWarpQuality,
    NodeEmotions,
    NodeMask,
    NodeGlasses,
    NodeEyes,
    NodeGaze,
    NodeMouth,
    AggNodeFaceShots,
    AggNodeFaceBestshot,
    AggNodeFaceBasicAttributes,
    AggNodeFaceDescriptor,
    AggNodeLivenessV1,
    AggNodeDeepfake,
    AggNodeBodyShots,
    AggNodeBodyBestshot,
    AggNodeBodyDescriptor,
    AggNodeBodyAttributes,
}

HEAD_POSE_FILTER_TARGETS = {
    node.name  # type: ignore[attr-defined]
    for node in [
        AggNodeFaceShots,
        AggNodeFaceBestshot,
        AggNodeFaceBasicAttributes,
        AggNodeFaceDescriptor,
        AggNodeLivenessV1,
        AggNodeDeepfake,
    ]
}


def targetNodes(targets: Targets):
    yield _NodeEstimationPreserver
    yield _NodeAggregationPreserver
    yield from {node for node in EXTRA_NODES if node.name in targets}  # type: ignore[attr-defined]


def mandatoryNodes(params: Params, targets: Targets):
    nodes: set = {NodeHumanTrackingEventPolicy}

    if set(targets) & HEAD_POSE_FILTER_TARGETS:
        if params.faceSamples.filters.headRoll is not None:
            nodes |= {NodeHeadpose}
        if params.faceSamples.filters.headPitch is not None:
            nodes |= {NodeHeadpose}
        if params.faceSamples.filters.headYaw is not None:
            nodes |= {NodeHeadpose}
        if params.faceSamples.filters.score is not None:
            nodes |= {NodeAGS}
    yield from nodes
