import asyncio
import io

from lunavl.sdk.image_utils.image import VLImage

from ..models import ImageRetainPolicy


def _toBytes(image: VLImage, config: ImageRetainPolicy) -> tuple[bytes, float]:
    """Convert image to binary. Returns binary image and scaling ratio."""

    with io.BytesIO() as buf:
        img = image.asPillow()
        ratio = 1
        if config.maxSize != 0 and max(img.size) > config.maxSize:
            ratio = config.maxSize / max(img.size)
            img = img.resize(tuple(map(lambda x: round(x * ratio), img.size)))
        img.save(buf, format=config.mimetype, quality=config.quality)
        return buf.getvalue(), ratio


async def toBytes(image: VLImage, config: ImageRetainPolicy) -> tuple[bytes, float]:
    """Convert image to binary, run in separate thread. Returns binary image and scaling ratio."""

    return await asyncio.to_thread(_toBytes, image, config)
