from luna_analytics_manager.templates.models import FrequencyParams, RateFrame, RateTime  # noqa: compat
from luna_analytics_manager.templates.sdk import SDKFrameCtx


class FramesFrequencyMixin:
    """Frame frequency node"""

    def __init__(self):
        self.skipUntilTs = 0
        self.lastFramePassed = 1

    def checkFramesUnit(self, frameNum, period):
        """Handle frames unit skip"""
        if frameNum == 1:
            return False
        if not (skip := (self.lastFramePassed + period) > frameNum):
            self.lastFramePassed = frameNum
        return skip

    def checkSecondsUnit(self, frameNum, frameTs, period):
        """Handle seconds unit skip"""
        if frameNum == 1 or frameTs >= self.skipUntilTs:
            self.skipUntilTs = frameTs + period
            return False
        return True

    def checkFrameSkip(self, frameCtx: SDKFrameCtx, ctx, rate=None) -> bool:
        """Handle frames skipping"""

        if rate is not None:
            frequency = rate
        elif ctx.params.rate is not None:
            frequency = ctx.params.rate
        else:
            return False

        match frequency.unit:
            case "frame":
                return self.checkFramesUnit(frameCtx.number, frequency.period)
            case "second":
                return self.checkSecondsUnit(frameCtx.number, frameCtx.timestamp, frequency.period)
            case _:
                raise ValueError(f"Frequency unit '{frequency.unit}' is unsupported")
